#include <IDT.h>
#include <IRQ.h>

void SetIntrGate(int A_iIntNum, void *A_pHandler)
{
	IDT[A_iIntNum].OffsetHigh = (unsigned short)( ((unsigned long)A_pHandler >> 16));
	IDT[A_iIntNum].Type = DESTYPE_INT;
	IDT[A_iIntNum].SegSelector = KERNEL_CODE;
	IDT[A_iIntNum].OffsetLow = (unsigned short)((unsigned long)A_pHandler & 0xffff);
}

void SetSystemGate(int A_iIntNum, void *A_pHandler)
{
	IDT[A_iIntNum].OffsetHigh = (unsigned short)((unsigned long)A_pHandler & 0xffff0000) >> 16;
	IDT[A_iIntNum].Type = DESTYPE_TRAP;
	IDT[A_iIntNum].SegSelector = KERNEL_CODE;
	IDT[A_iIntNum].OffsetLow = (unsigned short)((unsigned long)A_pHandler & 0xffff);
}

void SetTrapGate(int A_iIntNum, void *A_pHandler)
{
	IDT[A_iIntNum].OffsetHigh = (unsigned short)((unsigned long)A_pHandler & 0xffff0000) >> 16;
	IDT[A_iIntNum].Type = DESTYPE_TRAP;
	IDT[A_iIntNum].SegSelector = KERNEL_CODE;
	IDT[A_iIntNum].OffsetLow = (unsigned short)((unsigned long)A_pHandler & 0xffff);
}


#define DECLARE_IRQ(num)	\
	extern void HandleIRQ_##num(void);	\
	SetIntrGate(IRQ_VECTOR+num, HandleIRQ_##num)

void InitIDT(void)
{
	int nIndex = 0;
	
	/* initialize all IDT with null handler */
	for (nIndex = 0; nIndex < 255; nIndex++) {
		SetTrapGate(nIndex, UnidentifiedINTR);
	}

	/* int 0 : Devide Error Exception */
	SetTrapGate(0, DivideZero);
	/* int 1 : Debug Exception */
	SetTrapGate(1, Debug);
	/* int 2 : NMI interrupt */
	SetIntrGate(2, NMI);
	/* int 3 : Breakpoint exception */
	SetSystemGate(3, BreakPoint);
	/* int 4 : Overflow Exception */
	SetSystemGate(4, Overflow);
	/* int 5 : BOUND Exceeded Exception */
	SetSystemGate(5, BOUNDExceed);
	/* int 6 : Invalid Opcode Exception */
	SetTrapGate(6, InvalidOpcode);
	/* int 7 : Device Not Available Exception */
	SetTrapGate(7, DeviceDisable);
	/* int 8 : Double Fault Exception */
	SetTrapGate(8, DoubleFault);
	/* int 9 : Coprocessor Segment Overrun */
	SetTrapGate(9, CoprocessorSegOver);
	/* int 10 : Invalid TSS Exception */
	SetTrapGate(10, InvalidTSS);
	/* int 11 : Segment Not Present */
	SetTrapGate(11, SegNotPresent);
	/* int 12 : Stack Fault Exception */
	SetTrapGate(12, StackFault);
	/* int 13 : General Protection Exception */
	SetTrapGate(13, GeneralProtection);
	/* int 14 : Page Fault Exception */
	SetIntrGate(14, PageFault);
	/* int 15 : reserved for INTEL */
	SetTrapGate(15, INTELReserved);
	/* int 16 : x87 FPU Floating Point Error */
	SetTrapGate(16, FloatingPoint);
	/* int 17 : Alignment Check Exception */
	SetTrapGate(17, AlignCheck);
	/* int 18 : Machine Check Exception */
	SetTrapGate(18, MachineCheck);
	/* int 19 : SIMD Floationg Point Exception */
	SetTrapGate(19, SIMDFault);

	DECLARE_IRQ(0);	
	DECLARE_IRQ(1);	
	DECLARE_IRQ(2);	
	DECLARE_IRQ(3);	
	DECLARE_IRQ(4);	
	DECLARE_IRQ(5);	
	DECLARE_IRQ(6);	
	DECLARE_IRQ(7);	
	DECLARE_IRQ(8);	
	DECLARE_IRQ(9);	
	DECLARE_IRQ(10);	
	DECLARE_IRQ(11);	
	DECLARE_IRQ(12);	
	DECLARE_IRQ(13);	
	DECLARE_IRQ(14);	
	DECLARE_IRQ(15);	

	//SetSystemGate(SYSCALL_VECTOR, HANDLER_SysCall);
	
}


